{
  File:      Animate.pas
  Function:  Component TAnimation: Windows 95 Animation Control (see Win32.hlp).
             An animation control is a window that silently displays an
             Audio Video Interleaved (AVI) clip. An AVI clip is a series of
             bitmap frames like a movie. Although AVI clips can have sound,
             you cannot use such clips with animation controls. You can use
             only silent AVI clips.
  Source:    none
  Language:  Delphi 2.0
  Author:    Rudolph Velthuis
  Style:     Borland
  Copyright: (c) 1997 drs. Rudolph Velthuis
}

unit Animate;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  CommCtrl, DsgnIntf;

type
  TAVIResource = (
    arNone, arCopyFile, arMoveFile, arDeleteFile, arEmptyBin, arEmptyFolder,
    arSearchFile, arSearchFolder, arSearchHardware, arUser);

  TAnimation = class(TWinControl)
  private
    FBorderStyle: TBorderStyle;
    FFrame: Integer;
    FStartFrame: Integer;
    FEndFrame: Integer;
    FPlaying: Boolean;
    FAutoPlay: Boolean;
    FAnimation: string;
    FRepeats: Integer;
    FResource: TAVIResource;
    FTransparent: Boolean;
    FCenter: Boolean;
    procedure SetBorderStyle(Value: TBorderStyle);
    procedure SetStartFrame(Value: Integer);
    procedure SetAnimation(Value: string);
    procedure SetRepeats(Value: Integer);
    procedure SetFrame(Value: Integer);
    procedure SetEndFrame(Value: Integer);
    procedure SetPlaying(Value: Boolean);
    procedure SetAutoPlay(Value: Boolean);
    procedure SetTransparent(Value: Boolean);
    procedure SetCenter(Value: Boolean);
    procedure SetResource(Value: TAVIResource);
  protected
    procedure CreateParams(var Params: TCreateParams); override;
    procedure CreateWnd; override;
    procedure DestroyWnd; override;
    procedure CMCtl3DChanged(var Message: TMessage); message CM_CTL3DCHANGED;
    procedure SetFrames(AFrame, AStartFrame, AnEndFrame, ARepeats: Integer); virtual;
    procedure Open(AName: string);
    procedure Play(PlayFrom, PlayTo, ARepeats: Integer);
    procedure Stop;
    procedure Close;
    procedure Seek(AFrame: Integer);
  public
    constructor Create(AOwner: TComponent); override;
  published
    property Animation: string read FAnimation write SetAnimation;
    property AutoPlay: Boolean read FAutoPlay write SetAutoPlay default False;
    property BorderStyle: TBorderStyle read FBorderStyle write SetBorderStyle;
    property Ctl3D;
    property Center: Boolean read FCenter write SetCenter default True;
    property EndFrame: Integer read FEndFrame write SetEndFrame default -1;
    property Frame: Integer read FFrame write SetFrame default 0;
    property Playing: Boolean read FPlaying write SetPlaying stored True;
    property Repeats: Integer read FRepeats write SetRepeats default -1;
    property Resource: TAVIResource read FResource write SetResource default arNone;
    property StartFrame: Integer read FStartFrame write SetStartFrame default 0;
    property Transparent: Boolean read FTransparent write SetTransparent default True;
    property Visible;
  end;

  TAnimationProperty = class(TStringProperty)
    function GetAttributes: TPropertyAttributes; override;
    procedure Edit; override;
  end;

  TAnimationResourceProperty = class(TEnumProperty)
    function GetAttributes: TPropertyAttributes; override;
    procedure Edit; override;
  end;

procedure Register;

implementation

uses FAVISel;

{$R AVI.RES}

const
  AVIResources: array[TAVIResource] of string[16] = (
    '', 'CopyFile', 'CopyFiles', 'DeleteFile', 'EmptyBin', 'EmptyFolder',
    'SearchFile', 'SearchFolder', 'SearchHardware', '');

procedure Register;
begin
  RegisterComponents('VeltSoft', [TAnimation]);
  RegisterPropertyEditor(TypeInfo(string), TAnimation, 'Animation',
    TAnimationProperty);
  {
  RegisterPropertyEditor(TypeInfo(TAVIResource), TAnimation, 'Resource',
    TAnimationResourceProperty);
  }
end;

procedure TAnimation.SetBorderStyle(Value: TBorderStyle);
begin
  if Value <> FBorderStyle then
  begin
    FBorderStyle := Value;
    RecreateWnd;
  end;
end;

procedure TAnimation.CMCtl3DChanged(var Message: TMessage);
begin
  if NewStyleControls and (FBorderStyle = bsSingle) then
    RecreateWnd;
  inherited;
end;

procedure TAnimation.CreateParams(var Params: TCreateParams);
const
  BorderStyles: array[TBorderStyle] of Integer = (0, WS_BORDER);
  CenterStyles: array[Boolean] of Integer = (0, ACS_CENTER);
  TransparentStyles: array[Boolean] of Integer = (0, ACS_TRANSPARENT);
  AutoPlayStyles: array[Boolean] of Integer = (0, ACS_AUTOPLAY);
begin
  InitCommonControls;
  inherited CreateParams(Params);
  CreateSubClass(Params, ANIMATE_CLASS);
  with Params do
  begin
    Style := Style or BorderStyles[FBorderStyle] or CenterStyles[FCenter] or
               TransparentStyles[FTransparent] or AutoPlayStyles[FAutoPlay];
    if NewStyleControls and Ctl3D and (FBorderStyle = bsSingle) then
    begin
      Style := Style and not WS_BORDER;
      ExStyle := ExStyle or WS_EX_STATICEDGE;
    end;
  end;
end;

procedure TAnimation.Seek(AFrame: Integer);
begin
  Play(AFrame, AFrame, 1);
end;

procedure TAnimation.Close;
begin
  Open('');
end;

procedure TAnimation.Stop;
begin
  Perform(ACM_STOP, 0, 0);
  FPlaying := False;
end;

procedure TAnimation.Open(AName: string);
var
  P: PChar;
begin
  if AName = '' then
    P := nil
  else
    P := PChar(AName);
  if Bool(Perform(ACM_OPEN, 0, Longint(P))) and FAutoPlay then
    FPlaying := True;
end;

procedure TAnimation.SetAnimation(Value: string);
var
  I: TAVIResource;
begin
  if Value <> FAnimation then
  begin
    Close;
    Open(Value);
    FAnimation := Value;
    for I := Low(TAVIResource) to High(TAVIResource) do
      if UpperCase(AVIResources[I]) = UpperCase(Value) then
      begin
        FResource := I;
        Break;
      end
      else
        FResource := arUser;
  end;
  if FPlaying then
    Play(FStartFrame, FEndFrame, FRepeats);
end;

procedure TAnimation.Play(PlayFrom, PlayTo, ARepeats: Integer);
begin
  if Bool(Perform(ACM_PLAY, ARepeats, MakeLong(PlayFrom, PlayTo))) then
  begin
    FPlaying := True;
    if (PlayFrom = PlayTo) and (ARepeats = 1) then
    begin
      FFrame := PlayFrom;
      FPlaying := False;
    end
    else
    begin
      FStartFrame := PlayFrom;
      FEndFrame := PlayTo;
      FRepeats := ARepeats;
    end;
  end;
end;

procedure TAnimation.DestroyWnd;
begin
  inherited DestroyWnd;
end;

constructor TAnimation.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FCenter := True;
  FRepeats := -1;
  FFrame := 0;
  FStartFrame := 0;
  FEndFrame := -1;
  FPlaying := False;
  FAutoPlay := False;
  FTransparent := True;
  TabStop := False;
  Width := 40;
  Height := 40;
  FBorderStyle := bsNone;
end;

procedure TAnimation.CreateWnd;
begin
  inherited CreateWnd;
  if FAnimation <> '' then
    Open(FAnimation);
  if FPlaying then
    Play(FStartFrame, FEndFrame, FRepeats);
end;

procedure TAnimation.SetCenter(Value: Boolean);
begin
  if FCenter <> Value then
  begin
    FCenter := Value;
    RecreateWnd;
  end;
end;

procedure TAnimation.SetTransparent(Value: Boolean);
begin
  if FTransparent <> Value then
  begin
    FTransparent := Value;
    RecreateWnd;
  end;
end;

procedure TAnimation.SetAutoPlay(Value: Boolean);
begin
  if FAutoPlay <> Value then
  begin
    FAutoPlay := Value;
    RecreateWnd;
  end;
end;

procedure TAnimation.SetPlaying(Value: Boolean);
begin
  FPlaying := Value;
  if Value then
    Play(FStartFrame, FEndFrame, FRepeats)
  else
    Stop;
end;

procedure TAnimation.SetEndFrame(Value: Integer);
begin
  SetFrames(FFrame, FStartFrame, Value, FRepeats);
end;

procedure TAnimation.SetStartFrame(Value: Integer);
begin
  SetFrames(FFrame, Value, FStartFrame, FRepeats);
end;

procedure TAnimation.SetFrames(AFrame, AStartFrame, AnEndFrame, ARepeats: Integer);
begin
  FFrame := AFrame;
  FStartFrame := AStartFrame;
  FEndFrame := AnEndFrame;
  FRepeats := ARepeats;
  if (AFrame <> -1) or (AStartFrame = AnEndFrame) then
  begin
    Seek(AFrame);
    FPlaying := False;
  end
  else
  begin
    if FPlaying then
      Play(AStartFrame, AnEndFrame, ARepeats);
  end;
end;

procedure TAnimation.SetFrame(Value: Integer);
begin
  SetFrames(Value, FStartFrame, FEndFrame, FRepeats);
end;

procedure TAnimation.SetRepeats(Value: Integer);
begin
  SetFrames(FFrame, FStartFrame, FEndFrame, Value);
end;

procedure TAnimation.SetResource(Value: TAVIResource);
begin
  if FResource <> Value then
    SetAnimation(AVIResources[Value]);
end;

{ ... TAnimationProperty ... }

function TAnimationProperty.GetAttributes: TPropertyAttributes;
begin
  Result := [paDialog, paRevertable, paMultiSelect];
end;

procedure TAnimationProperty.Edit;
begin
  with TSelectAVIForm.Create(Application) do
  try
    Animation.Animation := GetValue;
    AnimationLabel.Caption := GetValue;
    if Execute then
      SetValue(Animation.Animation);
  finally
    Free;
  end;
end;

{ ... TAnimationResourceProperty ... }

function TAnimationResourceProperty.GetAttributes: TPropertyAttributes;
begin
  Result := [paDialog, paRevertable, paMultiSelect];
end;

procedure TAnimationResourceProperty.Edit;
begin
  with SelectAVIForm do
  begin
    Animation.Animation := Value;
    if Execute then
      SetValue(Animation.Animation);
  end;
end;

end.
